
# AWS Config and Suggested Systems Manager Remediations for any Jira Issue

_Note: this document is provided as a guide; it is not the canonical or most-recent set of instructions,
which are maintained externally at AWS, and should be preferred to these._


The Connector provides two fields that can be used and displayed for any issue.

* **AWS Config Linked Resources**:  this field allows any resource with an entry in AWS Config to
  have its AWS Config information displayed on the issue in Jira; the information can be expanded
  and information on relations also displayed; multiple AWS resources can be linked to an issue.   
    
* **AWS Systems Manager Automation Suggested Remediation**:  this field allows SSM automation documents
  to be recorded against an issue; these are then displayed as suggested ways to correct the issue;
  when a Jira user views the issue, they can see these suggested remediations and click to apply them;
  multiple suggested remediations can be attached to an issue
  
The two fields can be used individually but they work very well together:  when an incident is detected
on an AWS resource or set of resources, setting both allows a Jira user to see the Config information to
confirm or better understand the problem, apply remediations which fix common problems, and then confirm
in the Config information that the problem has been fixed.


## To add AWS fields to an existing issue

To use these fields:

1. The project or projects must be enabled for the Connector. This is done in
   **Admin -> Manage apps -> Connector settings**, as described elsewhere in the Connector setup guide.

Next configure the "screen" for the Jira project or projects:

2.  Go to **Admin -> Projects**, and open the project you wish to use these fields with.
3.  Select the issue type you wish to use in the menu at left.
4.  Select to view "Fields" in the top right (if not already selected). It should then show a list of fields enabled for the screen.
4.  Scroll to the bottom where there should be a textbox where you can type additional fields.  
    Start typing "AWS" then select the AWS field you want to use.
5.  Click "Add" to apply.
6.  Repeat the previous step for the other field if you wish to use it.
7.  Repeat these steps for each issue type you wish you use these fields with.
    Some issue types may share screens so the field may already be added for some.
8.  Repeat these steps for each project you wish to use these fields with.

It is important also to make a note of the field ID for the field or fields you are using.
This can be done by going to Admin -> Issues -> Custom fields and clicking Configure on each such field.
Inspect the URL that is opened to see the numeric field ID.  It should be a 5 digit number.

Alternatively, for any issue in a project where you’ve added the field (following the instructions above), 
the REST API at `<jira base URL>/rest/api/2/issue/PRJ-1/editmeta` 
(e.g. `http://localhost:2990/jira/rest/api/2/issue/PRJ-1/editmeta`) 
will include information on the fields and should contain an entry 
`customfield_#####: { ..., name: "AWS Config Linked Resources", ... }`, where `#####` is the numberic field ID.


## To set these fields on issues

Once these fields are enabled for projects and issue types, use the Jira REST API to create or update 
issues with values for these fields. This can be used from tools such as CloudWatch or AppDynamics or
Jenkins or a Systems Manager Automation Document (provided in the next section).

The REST API endpoint to update an issue is `<Jira-base-URL>/rest/api/2/issue/<issue-key>`
and the general schema to pass to set a value is:

    { "update": { 
        "customfield_<field-ID>": [ {
          "set": "<value>" 
        } ] 
    } }

See the examples below, or for more information on the REST API, refer to
https://developer.atlassian.com/server/jira/platform/updating-an-issue-via-the-jira-rest-apis-6848604/ .


### AWS Config Linked Resources

The **AWS Config Linked Resources** field should be set to the JSON string representation of a list of objects
(maps) corresponding to the linked resources, each with the following keys:

* `resourceId`: the ID of the resource in AWS Config 
* `resourceType`: the type of the resourcein AWS Config 
* `accountName`: the name/alias of the AWS account configured in Jira that should be used to access this resource
* `region`: the region where AWS Config should be accessed to get information on this resource

For example the following value would show information on the S3 bucket `my-bucket` in `eu-central-1`,
using the account and end-user credentials specified in Jira for the AWS Account identified in Jira as `MyAccount1`: 
    
    [ { "resourceId": "my-bucket", 
        "resourceType": "AWS::S3::Bucket", 
        "accountName": "MyAccount1", 
        "region": "eu-central-1" } ]


    
### AWS Systems Manager Automation Suggested Remediation

The **AWS Systems Manager Automation Suggested Remediation** field should be set to the JSON string representation of a list of objects
(maps) corresponding to the automation documents to suggest as remediations, each with the following keys:

* `documentName`: the name of the Systems Manager automation document 
* `description`: a description of the remediation to display in Jira; this may be different to the document description in AWS,
  and might explain why it is a good remediation for the issue where this is being set  
* `accountName`: the name/alias of the AWS account configured in Jira that should be used to access this resource
* `region`: the region where AWS Config should be accessed to get information on this resource

For example the following value would suggest the `AWS-DisableS3BucketPublicReadWrite`
automation document, with a nice description to show in Jira, to be applied in `eu-central-1`,
using the account and end-user credentials specified in Jira for the AWS Account identified in Jira as `MyAccount1`: 
    
    [ { "documentName": "AWS-DisableS3BucketPublicReadWrite", 
        "description": "This will make the bucket private, resolving the issue.", 
        "accountName": "MyAccount1", 
        "region": "eu-central-1" } ]


### Scripting Field Creation

As an example, the following bash script using curl will link the above-noted resource to an issue and attach a suggested remediation. 
The values used below assume Jira is at `localhost:2990/jira` with login `admin:admin`, the issue is `PRJ-1`,
and the field IDs are 10011 (AWS Config linked resources) and 10010 (suggested remediation);
these should be changed to reflect your environment.

```
# set these corresponding to your environment and issue
JIRA_BASE_URL=http://localhost:2990/jira
JIRA_USER_PASS=admin:admin
ISSUE_KEY=PRJ-1

# set the field ID and edit the JSON record for an AWS config resource to link
CUSTOM_FIELD_ID=customfield_10011
cat > value.json << EOF
    [ { "resourceId": "my-bucket", 
        "resourceType": "AWS::S3::Bucket", 
        "accountName": "MyAccount1", 
        "region": "eu-central-1" } ]
EOF

# define a helper function to escape the JSON
json_escape () { 
    printf '%s' "$1" | python -c \
      'import json,sys; print(json.dumps(sys.stdin.read()))'
}

# make the REST call to set the AWS Config Linked Resource field
curl -v -D- -X PUT  -H "Content-Type: application/json" \
  --data '{ "update": { "'${CUSTOM_FIELD_ID}'": [ {"set": '"$(
     json_escape "$(cat value.json)")"' } ] } }' \
  -u admin:admin ${JIRA_BASE_URL}/rest/api/2/issue/${ISSUE_KEY}

# set the field ID and edit the JSON record for a suggested remediation to attach
CUSTOM_FIELD_ID=customfield_10010
cat > value.json << EOF
    [ { "documentName": "AWS-DisableS3BucketPublicReadWrite", 
        "description": "This will make the bucket private, resolving the issue.", 
        "accountName": "MyAccount1", 
        "region": "eu-central-1" } ]
EOF

# make the REST call to set the AWS Systems Manager Automation Suggested Remediations field
curl -v -D- -X PUT  -H "Content-Type: application/json" \
  --data '{ "update": { "'${CUSTOM_FIELD_ID}'": [ {"set": '"$(
     json_escape "$(cat value.json)")"' } ] } }' \
  -u ${JIRA_USER_PASS} ${JIRA_BASE_URL}/rest/api/2/issue/${ISSUE_KEY}
```

The issue should then show AWS Config for the bucket and a suggested remediation to make it private.
 

## Creating Issues with Suggestions and a Linked AWS Resource from AWS Systems Manager

A Systems Manager Automation Document is provided which can automatically create a Jira issue
with the fields set to have a linked AWS resource and up to three suggested remediation documents.

To install this automation document, you will need two files:

* JSDConnector-CreateRemediationIssue.ssmdoc.yaml
* JSDConnector-function.zip

Follow these steps:

1. Upload the file `JSDConnector-function.zip` to a bucket. The following command will do this (replacing `${BUCKET}` with the appropriate bucket):

        aws s3 cp JSDConnector-function.zip s3://${BUCKET}/function.zip

2. Create the Systems Manager Automation Document, called `JSDConnector-CreateRemediationIssue`, with the contents taken from the file `JSDConnector-CreateRemediationIssue.ssmdoc.yaml` and an attachment `Key=SourceUrl,Values=s3://${BUCKET}/`, using the bucket name from the previous step as `${BUCKET}`. The following command will do this (replacing `${BUCKET}`):

        aws ssm create-document --name "JSDConnector-CreateRemediationIssue" --content "file://JSDConnector-CreateRemediationIssue.ssmdoc.yaml" --document-type "Automation" --document-format "YAML" --attachments "Key=SourceUrl,Values=s3://${BUCKET}/" 


Once installed, use it as you would any other Systems Manager automation document,
filling out the parameters and running it.  Note this requires many of the same parameters
as described previously in order to connect to Jira.

You should then see an issue in Jira with AWS Config information and the suggested remediation shown.


## Demo: Automatically Creating Issues for IT Lifecycle Management, fixing non-compliant public buckets

Once the fields are enabled to an issue and the Systems Manager Automation Document is created,
you can set up rules to automatically create Jira issues for common problem categories in AWS
and include suggested remediations to make it easy for a Jira operator to see problems and fix them.

This demo will create a Config Rule is AWS which detects public buckets and makes it possible for
a Jira operator to disable public access directly from Jira.


### Pre-requisites

First set up pre-requisites, roles for the automation and lambda to execute as,
and the Jira password as a secure string in Systems Manager Parameter Store.

First, store the Jira password securely in Parameter Store: 

1. Open the AWS Console and go to **Systems Manager -> Parameter Store**.
2. Click "Create parameter".
3. Set the name as `jira_password`.
4. Set the type as "SecureString".
5. Set the value as the password for the Jira user to be used to create issues.
6. Click "Create parameter" to save this.


#### Installing

A CloudFormation template is provided to assist setting up the role and config rule:

* `JSDConnector-CreateRemediationIssue-MakePublicBucketsPrivateConfigRule.cfn.yaml`

Install this, setting the following parameters:

  * **JiraURL**: the base URL to your Jira, such that appending `/rest/...` after it accesses the REST API
  * **JiraUsername**: the username to use to log in to Jira (with the password specified in `jira_password`) 
  * **SSMParameterName**: `jira_password` (the parameter containing the Jira password)
  * **ProjectKey**: the key of the project (the token before the `-` in an issue), e.g. `PRJ` 
  * **IssueTypeName**: this must exactly match the name of the issue type on the project in Jira
  * **JiraAwsAccountName**: the name of the AWS Account as configured in the Connector in Jira
  * **JiraAwsAccountRegion**: the region where this violating resource is found, e.g. `us-east-1`
  * **JiraAwsResourceFieldId**: enter the field ID of the AWS Config Linked Resources field in Jira, e.g. `customfield_10011` 
  * **JiraRemediationsFieldId**: enter the field ID of the AWS Systems Manager Automation Suggested Remediation field in Jira, e.g. `customfield_10010` 


### Running the Config Rule

The rule will run automatically with the period specified.
To see it in action immediately, first create a public bucket.
Next open the Config Rule in AWS Config and click "Re-evaluate".
The rule and the automation may take a short while to run,
but within a few minutes you should see a new issue in Jira with
AWS Config information for the bucket which is in violation and
suggesting the DisableS3BucketPublicReadWrite automation document as a remediation.

